# Default locations
BUILDFLAGS_PARSER?=    /usr/local/share/bsdadminscripts2/buildflags.awk
BUILDFLAGS_CONF?=      /usr/local/etc/buildflags.conf
BUILDFLAGS_USER?=      ${HOME}/.buildflags.conf
BUILDFLAGS_TMP?=       /tmp/buildflags.tmp.mk.${USER}

BUILDFLAGS_DISTCC?=    /usr/local/bin/distcc
BUILDFLAGS_CCACHE?=    /usr/local/bin/ccache

# Parse configurations
.if exists(${BUILDFLAGS_CONF}) && exists(${BUILDFLAGS_USER})
BUILDFLAGS!=           test    "${BUILDFLAGS_TMP}" -nt "${BUILDFLAGS_CONF}" \
                            -a "${BUILDFLAGS_TMP}" -nt "${BUILDFLAGS_USER}" \
                       || "${BUILDFLAGS_PARSER}" "${BUILDFLAGS_USER}" \
                                                 "${BUILDFLAGS_CONF}" \
                          > "${BUILDFLAGS_TMP}"; echo
.elif exists(${BUILDFLAGS_CONF})
BUILDFLAGS!=           test    "${BUILDFLAGS_TMP}" -nt "${BUILDFLAGS_CONF}" \
                       || "${BUILDFLAGS_PARSER}" "${BUILDFLAGS_CONF}" \
                          > "${BUILDFLAGS_TMP}"; echo
.elif exists(${BUILDFLAGS_USER})
BUILDFLAGS!=           test    "${BUILDFLAGS_TMP}" -nt "${BUILDFLAGS_USER}" \
                       || "${BUILDFLAGS_PARSER}" "${BUILDFLAGS_USER}" \
                          > "${BUILDFLAGS_TMP}"; echo
.endif

# Include generated make file
.sinclude "${BUILDFLAGS_TMP}"

# Deprecated, because USE_ variables are not supposed to be user-defined
.if defined(USE_CCACHE)
.warning "buildflags.mk: USE_CCACHE is deperecated in favour of WITH_CCACHE_BUILD"
.endif
.if defined(USE_DISTCC)
.warning "buildflags.mk: USE_DISTCC is deperecated"
.endif

# Use ccache and distcc.
.if defined(USE_CCACHE) && !${CC:M*ccache*} && exists(${BUILDFLAGS_CCACHE}) && defined(USE_DISTCC) && !${CC:M*distcc*} && exists(${BUILDFLAGS_DISTCC})
CC:=			env CCACHE_PREFIX=${BUILDFLAGS_DISTCC} ${BUILDFLAGS_CCACHE} ${CC}
CPP:=			env CCACHE_PREFIX=${BUILDFLAGS_DISTCC} ${BUILDFLAGS_CCACHE} ${CPP}
CXX:=			env CCACHE_PREFIX=${BUILDFLAGS_DISTCC} ${BUILDFLAGS_CCACHE} ${CXX}
# Use distcc.
.elif defined(USE_DISTCC) && !${CC:M*distcc*} && exists(${BUILDFLAGS_DISTCC}) && !${CC:M*ccache*}
CC:=			${BUILDFLAGS_DISTCC} ${CC}
CPP:=			${BUILDFLAGS_DISTCC} ${CPP}
CXX:=			${BUILDFLAGS_DISTCC} ${CXX}
# Use ccache.
.elif defined(USE_CCACHE) && !${CC:M*ccache*} && exists(${BUILDFLAGS_CCACHE}) && !${CC:M*distcc*}
CC:=			${BUILDFLAGS_CCACHE} ${CC}
CPP:=			${BUILDFLAGS_CCACHE} ${CPP}
CXX:=			${BUILDFLAGS_CCACHE} ${CXX}
.endif

# Activate normal parallel builds.
.if defined(THREADS)
.MAKEFLAGS:		-j${THREADS}
.endif
